from typing import Protocol

from ytrssil.datatypes import Video


class Client(Protocol):
    def fetch(self) -> None:  # pragma: no cover
        ...

    def register(self) -> None:  # pragma: no cover
        ...

    def subscribe_to_channel(
        self,
        channel_id: str,
    ) -> None:  # pragma: no cover
        ...

    def get_new_videos(self) -> list[Video]:  # pragma: no cover
        ...

    def get_watched_videos(self) -> list[Video]:  # pragma: no cover
        ...

    def mark_video_as_watched(self, video_id: str) -> None:  # pragma: no cover
        ...

    def mark_video_as_unwatched(
        self,
        video_id: str,
    ) -> None:  # pragma: no cover
        ...



from dataclasses import dataclass
from datetime import datetime
from typing import Optional


@dataclass
class Video:
    video_id: str
    name: str
    channel_name: str
    timestamp: datetime
    watch_timestamp: Optional[datetime] = None

    def __str__(self) -> str:
        return f'{self.channel_name} - {self.name} - {self.video_id}'


@dataclass
class Channel:
    channel_id: str
    name: str

    def __str__(self) -> str:
        return f'{self.name} - {self.channel_id}'


@dataclass
class User:
    username: str
    password: str



from setuptools import setup  # type: ignore


with open('readme.md', 'r') as f:
    long_description = f.read()

with open('requirements.txt', 'r') as f:
    required = f.read().splitlines()


setup(
    name='ytrssil',
    author='Pavle Portic',
    author_email='git@theedgeofrage.com',
    description=(
        'Subscribe to YouTube RSS feeds and keep track of watched videos'
    ),
    long_description=long_description,
    long_description_content_type='text/markdown',
    license_files=('LICENSE',),
    url='https://gitea.theedgeofrage.com/TheEdgeOfRage/ytrssil',
    version_config=True,
    setup_requires=['setuptools-git-versioning'],
    packages=['ytrssil'],
    package_data={'': ['py.typed']},
    include_package_data=True,
    install_requires=required,
    entry_points={
        'console_scripts': [
            'ytrssil = ytrssil.cli:main',
        ],
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX :: Linux',
    ],
)



import json
import os
from dataclasses import dataclass
from typing import Literal


@dataclass
class Configuration:
    username: str
    password: str
    api_url: str = 'https://ytrssil.theedgeofrage.com'
    max_res: Literal['480', '720', '1080', '1440', '2160'] = '1440'

    @property
    def mpv_options(self) -> list[str]:
        return [
            '--no-terminal',
            f'--ytdl-format=bestvideo[height<=?{self.max_res}]+bestaudio/best',
        ]


def load_config() -> Configuration:
    config_prefix: str
    try:
        config_prefix = os.environ['XDG_CONFIG_HOME']
    except KeyError:
        config_prefix = os.path.expanduser('~/.config')

    config_path: str = os.path.join(config_prefix, 'ytrssil', 'config.json')
    with open(config_path) as f:
        config_data = json.load(f)

    return Configuration(**config_data)
